/*
 * Copyright 2016 (c) Anna Schumaker.
 */
#include <core/idle.h>
#include <core/string.h>
#include <gui/filter.h>
#include <gui/model.h>
#include <gui/playlist.h>
#include <gui/treeview.h>
#include <gui/sidebar.h>

static void (*update_size[PL_MAX_TYPE])(struct playlist *) = {
	[PL_SYSTEM]  = gui_pl_system_update,
	[PL_ARTIST]  = gui_pl_artist_update,
	[PL_LIBRARY] = gui_pl_library_update,
	[PL_USER]    = gui_pl_user_update,
};

static void (*select_playlist[PL_MAX_TYPE])(struct playlist *) = {
	[PL_SYSTEM]  = gui_pl_system_select,
	[PL_ARTIST]  = gui_pl_artist_select,
	[PL_LIBRARY] = gui_pl_library_select,
	[PL_USER]    = gui_pl_user_select,
};

static inline void __gui_playlist_update_size(struct playlist *playlist)
{
	update_size[playlist->pl_type](playlist);
}

static void *__gui_playlist_init(struct queue *queue, void *data)
{
	struct playlist *playlist = (struct playlist *)data;

	if (playlist->pl_type == PL_ARTIST)
		gui_pl_artist_add(playlist);
	return playlist;
}

static void __gui_playlist_deinit(struct queue *queue)
{
	gui_filter_clear_search(queue->q_private);
}

static void __gui_playlist_added(struct queue *queue, unsigned int row)
{
	gui_model_add(queue->q_private, row);
	__gui_playlist_update_size(queue->q_private);
}

static void __gui_playlist_removed(struct queue *queue, unsigned int row)
{
	gui_model_remove(queue->q_private, row);
	__gui_playlist_update_size(queue->q_private);
}

static void __gui_playlist_cleared(struct queue *queue, unsigned int n)
{
	gui_model_clear(queue->q_private, n);
	__gui_playlist_update_size(queue->q_private);
}

static void __gui_playlist_updated(struct queue *queue, unsigned int n)
{
	gui_model_update(queue->q_private, n);
}

static bool __gui_playlist_erase(struct queue *queue, struct track *track)
{
	struct playlist *playlist = queue->q_private;
	enum playlist_type_t type = playlist->pl_type;
	const gchar         *name = playlist->pl_name;

	switch (type) {
	case PL_SYSTEM:
		if (string_match(name, "Collection")) {
			playlist_add(type, "Hidden", track);
			break;
		} else if (!string_match(name, "Favorites") &&
			 !string_match(name, "Hidden")    &&
			 !string_match(name, "Queued Tracks"))
			break;
	case PL_USER:
		playlist_remove(type, name, track);
	default:
		break;
	};

	return false;
}


struct queue_ops playlist_ops = {
	.qop_init    = __gui_playlist_init,
	.qop_deinit  = __gui_playlist_deinit,
	.qop_added   = __gui_playlist_added,
	.qop_erase   = __gui_playlist_erase,
	.qop_removed = __gui_playlist_removed,
	.qop_cleared = __gui_playlist_cleared,
	.qop_updated = __gui_playlist_updated,
};


static void __gui_playlist_add_selected_to(struct playlist *playlist)
{
	GList *cur, *list = NULL;
	struct track *track;

	if (!playlist)
		return;

	list = gui_treeview_list_selected_tracks();
	cur  = g_list_first(list);
	while (cur) {
		track = (struct track *)cur->data;
		playlist_add(playlist->pl_type, playlist->pl_name, track);
		cur = g_list_next(cur);
	}
	g_list_free(list);
}

void __gui_playlist_add_favorites(GtkMenuItem *item, gpointer data)
{
	__gui_playlist_add_selected_to(playlist_get(PL_SYSTEM, "Favorites"));
}

void __gui_playlist_add_hidden(GtkMenuItem *item, gpointer data)
{
	__gui_playlist_add_selected_to(playlist_get(PL_SYSTEM, "Hidden"));
}

void __gui_playlist_add_user(GtkMenuItem *item, gpointer data)
{
	__gui_playlist_add_selected_to(gui_pl_user_add_dialog());
}

void __gui_playlist_add_queued(GtkMenuItem *item, gpointer data)
{
	__gui_playlist_add_selected_to(playlist_get(PL_SYSTEM, "Queued Tracks"));
}

void __gui_playlist_delete(GtkMenuItem *item, gpointer data)
{
	struct playlist *playlist = gui_model_get_playlist();
	GList *cur, *list = NULL;
	struct track *track;

	if (!playlist)
		return;

	list = gui_treeview_list_selected_tracks();
	cur  = g_list_first(list);
	while (cur) {
		track = (struct track *)cur->data;
		queue_erase_track(&playlist->pl_queue, track);
		cur = g_list_next(cur);
	}
	g_list_free(list);
}

void __gui_playlist_keypress(GtkTreeView *treeview, GdkEventKey *event,
			     gpointer data)
{
	switch (event->keyval) {
	case GDK_KEY_f:
		__gui_playlist_add_favorites(NULL, NULL);
		break;
	case GDK_KEY_h:
		__gui_playlist_add_hidden(NULL, NULL);
		break;
	case GDK_KEY_p:
		__gui_playlist_add_user(NULL, NULL);
		break;
	case GDK_KEY_q:
		__gui_playlist_add_queued(NULL, NULL);
		break;
	case GDK_KEY_Delete:
		__gui_playlist_delete(NULL, NULL);
	default:
		break;
	}
}

void __gui_playlist_row_activated(GtkTreeView *treeview, GtkTreePath *path,
				  GtkTreeViewColumn *col, gpointer data)
{
	struct playlist *prev = playlist_cur();

	gui_sidebar_filter_path_select(path);
	__gui_playlist_update_size(prev);
	__gui_playlist_update_size(playlist_cur());
}

void __gui_playlist_row_collapsed(GtkTreeView *treeview, GtkTreeIter *iter,
				  GtkTreePath *path,     gpointer data)
{
	gui_sidebar_filter_row_expanded(iter, false);
}

void __gui_playlist_row_expanded(GtkTreeView *treeview, GtkTreeIter *iter,
				 GtkTreePath *path,     gpointer data)
{
	gui_sidebar_filter_row_expanded(iter, true);
}

bool __gui_playlist_init_idle()
{
	struct playlist *playlist = playlist_cur();
	GtkTreeModel    *filter   = GTK_TREE_MODEL(gui_sidebar_filter());
	GtkTreeIter      iter;

	gtk_tree_model_get_iter_first(filter, &iter);
	do {
		gui_sidebar_filter_set_expand(&iter);
	} while (gtk_tree_model_iter_next(filter, &iter));

	select_playlist[playlist->pl_type](playlist);
	return true;
}

void gui_playlist_init()
{
	gui_pl_system_init();
	gui_pl_artist_init();
	gui_pl_user_init();
	gui_pl_library_init();

	idle_schedule(IDLE_SYNC, __gui_playlist_init_idle, NULL);
}
